/*
 * (C) COPYRIGHT 2013 CRZ
 *
 * File Name : mcu_comm_command.c
 * Author    : POOH
 * Version   : V1.0
 */

/* includes */

#include "hw_config.h"
#include "bluetooth_command.h"

/* Defines */

/* Variables */

uint8_t g_mcu_comm_buffer_tx[MCU_COMM_BUFFER_SIZE];

/* Static Functions */

static void make_cmd_common_send(void)
{
    uint8_t i;

    g_mcu_comm_buffer_tx[MCU_COMM_BUFFER_SIZE - 2] = MCU_COMM_TAIL_CR;
    g_mcu_comm_buffer_tx[MCU_COMM_BUFFER_SIZE - 1] = MCU_COMM_TAIL_LF;

    Uart_2_Comm_SendData_Chars(g_mcu_comm_buffer_tx, MCU_COMM_BUFFER_SIZE);

#ifdef MCU_COMM_DEBUG_ON
    /* Print All */
    printf("%s\n", g_mcu_comm_buffer_tx);
    for(i = 0; i < MCU_COMM_BUFFER_SIZE; i++)
    {
        printf("%02X ", g_mcu_comm_buffer_tx[i]);
    }
    printf("\n");
#endif
}

#define CMD_MODULE_STATE "ATC80=03"
void make_cmd_module_state(void)
{
    memset(g_mcu_comm_buffer_tx, 0, sizeof(g_mcu_comm_buffer_tx));
    memcpy(g_mcu_comm_buffer_tx, CMD_MODULE_STATE, sizeof(CMD_MODULE_STATE));

    make_cmd_common_send();
}

#define CMD_MODULE_SENSITIVITY "ATC06=00"
void make_cmd_module_sensitivity(void)
{
    memset(g_mcu_comm_buffer_tx, 0, sizeof(g_mcu_comm_buffer_tx));
    memcpy(g_mcu_comm_buffer_tx, CMD_MODULE_SENSITIVITY, sizeof(CMD_MODULE_SENSITIVITY));

    make_cmd_common_send();
}

#define CMD_FIRMWARE_VERSION "ATC02=04"
void make_cmd_firmware_version(void)
{
    memset(g_mcu_comm_buffer_tx, 0, sizeof(g_mcu_comm_buffer_tx));
    memcpy(g_mcu_comm_buffer_tx, CMD_FIRMWARE_VERSION, sizeof(CMD_FIRMWARE_VERSION));

    make_cmd_common_send();
}

// crazyboy 2015/02/03
#define CMD_USER_FRIENDLY_NAME "ATC02=01"
void make_cmd_user_friendly_name(char *user_friendly_name)
{
    int cmd_size = sizeof(CMD_USER_FRIENDLY_NAME);
    printf("%s cmd_size = %d user friendly name : %s\n", __func__, cmd_size, user_friendly_name);
    memset(g_mcu_comm_buffer_tx, 0, sizeof(g_mcu_comm_buffer_tx));
    memcpy(g_mcu_comm_buffer_tx, CMD_USER_FRIENDLY_NAME, cmd_size);
    g_mcu_comm_buffer_tx[cmd_size-1] = strlen(user_friendly_name);
    memcpy(g_mcu_comm_buffer_tx+cmd_size, user_friendly_name, strlen(user_friendly_name));
    make_cmd_common_send();
}

// crazyboy 2015/02/06
#define CMD_PIN_CODE	"ATC02=00"
void make_cmd_pin_code(char *pin_code)
{
    int cmd_size = sizeof(CMD_PIN_CODE);
    printf("%s cmd_size = %d pin code : %s\n", __func__, cmd_size, pin_code);
    memset(g_mcu_comm_buffer_tx, 0, sizeof(g_mcu_comm_buffer_tx));
    memcpy(g_mcu_comm_buffer_tx, CMD_PIN_CODE, cmd_size);
    g_mcu_comm_buffer_tx[cmd_size-1] = strlen(pin_code);
    memcpy(g_mcu_comm_buffer_tx+cmd_size, pin_code, strlen(pin_code));
    make_cmd_common_send();
}

#define CMD_INQUIRY_START  "ATC00=02"
void make_cmd_inquiry_start(void)
{
    int cmd_size = sizeof(CMD_INQUIRY_START);
    memset(g_mcu_comm_buffer_tx, 0, sizeof(g_mcu_comm_buffer_tx));
    memcpy(g_mcu_comm_buffer_tx, CMD_INQUIRY_START, cmd_size);
    g_mcu_comm_buffer_tx[cmd_size - 1] = 0x02;
    g_mcu_comm_buffer_tx[cmd_size]     = 0x30; /* 0x30 is decimal 48, 48 * 1.28 = 61.44 seconds is max */
    g_mcu_comm_buffer_tx[cmd_size + 1] = 0x14; /* 0x14 is decimal 20, max 20 devices */
    
    make_cmd_common_send();
}

#define CMD_INQUIRY_STOP "ATC00=03"
void make_cmd_inquiry_stop(void)
{
    int cmd_size = sizeof(CMD_INQUIRY_STOP);
    memset(g_mcu_comm_buffer_tx, 0, sizeof(g_mcu_comm_buffer_tx));
    memcpy(g_mcu_comm_buffer_tx, CMD_INQUIRY_STOP, cmd_size);
    make_cmd_common_send();
}

uint8_t g_slave_bt_address_tmp[6];

#define CMD_SPP_CONNECT  "ATC80=00"
void make_cmd_spp_connect(void)
{
    int cmd_size = sizeof(CMD_SPP_CONNECT);
    memset(g_mcu_comm_buffer_tx, 0, sizeof(g_mcu_comm_buffer_tx));
    memcpy(g_mcu_comm_buffer_tx, CMD_SPP_CONNECT, cmd_size);
    g_mcu_comm_buffer_tx[cmd_size - 1] = 0x06;
    memcpy(&(g_mcu_comm_buffer_tx[cmd_size]), g_slave_bt_address_tmp, 6);
    
    make_cmd_common_send();

    Delay(150);
}

#define RES_PIN_CODE  "ATC01=01"
void make_res_pin_code(void)
{
    int cmd_size = sizeof(RES_PIN_CODE);
    memset(g_mcu_comm_buffer_tx, 0, sizeof(g_mcu_comm_buffer_tx));
    memcpy(g_mcu_comm_buffer_tx, RES_PIN_CODE, cmd_size);

    /* PIN Code "1234" */
    g_mcu_comm_buffer_tx[cmd_size - 1] = 0x04;
    memcpy(&(g_mcu_comm_buffer_tx[cmd_size]), "1234", 4);
    
    make_cmd_common_send();
}

#define CMD_SPP_DISCONNECT  "ATC80=02"
void make_cmd_spp_disconnect(void)
{
    int cmd_size = sizeof(CMD_SPP_DISCONNECT);
    memset(g_mcu_comm_buffer_tx, 0, sizeof(g_mcu_comm_buffer_tx));
    memcpy(g_mcu_comm_buffer_tx, CMD_SPP_DISCONNECT, cmd_size);
    
    make_cmd_common_send();

    Delay(150);
}

/* Discoverable Start */

#define CMD_DISCOVERABLE_START  "ATC00=01"
void make_cmd_discoverable_start(void)
{
    int cmd_size = sizeof(CMD_DISCOVERABLE_START);
    memset(g_mcu_comm_buffer_tx, 0, sizeof(g_mcu_comm_buffer_tx));
    memcpy(g_mcu_comm_buffer_tx, CMD_DISCOVERABLE_START, cmd_size);
    
    make_cmd_common_send();
}

