/*
 * (C) COPYRIGHT 2015 CRZ
 *
 * File Name : test_BT200.c
 * Author    : 
 * Version   : V1.0
 * Date      : 04/29/2015
 */

/* includes */

#include "hw_config.h"
#include "global_types.h"
#include "test_BT200.h"
#include "uart_queue.h"
#include "bluetooth_command.h"

/* functions */

/* PA4 (Reset) Output */
void Reset_BT200(void)
{
    GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
    Delay(100);
    GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
    Delay(200);
}

void ModeChange_PartronBT_to_Stream(void)
{
#if 0
    GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN);
    Delay(10);
#endif
    GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN);
    Delay(150);
}

void ModeChange_PartronBT_to_Normal(void)
{
#if 0
    GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN);
    Delay(10);
#endif
    GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN);
    Delay(150);
}

void Init_BT200(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    GPIO_InitStructure.GPIO_Pin   = GPIO_MCU_PA_4_PIN | GPIO_MCU_PA_6_PIN;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
    GPIO_Init(GPIO_MCU_PA, &GPIO_InitStructure);

    Reset_BT200();

    /* PA5 - PIO2 input mode */
    GPIO_InitStructure.GPIO_Pin   = GPIO_MCU_PA_5_PIN;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
    GPIO_Init(GPIO_MCU_PA, &GPIO_InitStructure);

    ModeChange_PartronBT_to_Normal();
}

#define BT_RX_BUFFER_SIZE 256

extern uint8_t g_slave_bt_address_tmp[6];

void Print_Receive_Data(uint8_t * rx_data_p, int rx_size)
{
    int i;

    printf("RX Size: %d, RX Data: ", rx_size);
    for(i = 0; i < rx_size; i ++)
    {
        if(0 == rx_data_p[i])
            printf("%c", ' ');
        else if((MCU_COMM_TAIL_CR != rx_data_p[i]) && (MCU_COMM_TAIL_LF != rx_data_p[i]))
            printf("%c", rx_data_p[i]);
    }
    printf("\n");

    for(i = 0; i < rx_size; i ++)
    {
        printf("%02X ", rx_data_p[i]);
    }
    printf("\n");
}

void Process_Receive_Data(uint8_t * rx_data_p)
{
    int i;

    if(0 == strncmp((char *)rx_data_p, (char *)RSP_INQUIRY_RESULT, sizeof(RSP_INQUIRY_RESULT) - 1))
    {
        printf("<< Receive Inquiry Result >>\n");

        memset(g_slave_bt_address_tmp, 0, sizeof(g_slave_bt_address_tmp));
        memcpy((char *)g_slave_bt_address_tmp, (char *)(&(rx_data_p[10])), 6);

        printf("Slave BT Addr: ");
        for(i = 0; i < 6; i ++)
        {
            printf("%02X ", g_slave_bt_address_tmp[i]);
        }
        printf("\n");
    }

    if(0 == strncmp((char *)rx_data_p, (char *)REQ_REMOTE_PIN_CODE, sizeof(REQ_REMOTE_PIN_CODE) - 1))
    {
        printf("<< Receive Request Remote PIN Code >>\n");
        make_res_pin_code();
    }
}

void test_rx_process(void)
{
    int i;
    uint8_t rx_data_prev = 0;
    uint8_t rx_data[BT_RX_BUFFER_SIZE];

    if(FALSE == Uart_2_Queue_Comm_Is_Empty())
    {
        rx_data_prev = 0;
        memset(rx_data, 0, sizeof(rx_data));
        for(i = 0; i < (BT_RX_BUFFER_SIZE - 1); i ++)
        {
            rx_data[i] = Uart_2_Queue_Comm_DeQueue();
            Delay(20);

            if(TRUE == Uart_2_Queue_Comm_Is_Empty())
            {
                i ++;
                break;
            }

            if((MCU_COMM_TAIL_CR == rx_data_prev) && (MCU_COMM_TAIL_LF == rx_data[i]))
            {
                i ++;
                break;
            }
            else
            {
                rx_data_prev = rx_data[i];
            }
        }

        Print_Receive_Data(rx_data, i);
        Process_Receive_Data(rx_data);
    }
}

static bool g_print_menu_control_flag_master = TRUE;

static void print_menu_command_master(void)
{
    if(TRUE == g_print_menu_control_flag_master)
    {
        printf("\n---------------------\n");
        printf("Press menu key - Master Mode\n");
        printf("---------------------\n");
        printf("0> Reset BT\n");
        printf("---------------------\n");
        printf("1> Normal Mode\n");
        printf("2> Stream Mode\n");
        printf("---------------------\n");
        printf("5> AT CMD: state\n");
        printf("7> AT CMD: firmware version\n");
        printf("---------------------\n");
        printf("8> Send Data: ABCDEFGH01234\n");
        printf("---------------------\n");
        printf("a> AT CMD: inquiry start\n");
        printf("b> AT CMD: inquiry stop\n");
        printf("c> AT CMD: spp connect\n");
        printf("d> AT CMD: spp disconnect\n");
        printf("---------------------\n");
    }

    g_print_menu_control_flag_master = FALSE;
}

void Test_BT200_Master(void)
{
    uint8_t ch;

    DEBUG_MSG_FUNC_START;
    
    while(1)
    {
        print_menu_command_master();

        if(FALSE == UartQueue_Serial_Is_Empty())
        {
            ch = UartQueue_Serial_DeQueue();
            printf("%c is selected\n\n", (char)ch);

            switch((char)ch)
            {
            case '0':
                Reset_BT200();
                break;

            case '1':
                ModeChange_PartronBT_to_Normal();
                break;

            case '2':
                ModeChange_PartronBT_to_Stream();
                break;

            case '5':
                make_cmd_module_state();
                break;

            case '7':
                make_cmd_firmware_version();
                break;

            case '8':
                Uart_2_Comm_SendData_Chars("ABCDEFGH01234", sizeof("ABCDEFGH01234"));
                break;

            case 'a':
                make_cmd_inquiry_start();
                break;

            case 'b':
                make_cmd_inquiry_stop();
                break;

            case 'c':
                make_cmd_spp_connect();
                ModeChange_PartronBT_to_Stream();
                break;

            case 'd':
                ModeChange_PartronBT_to_Normal();
                make_cmd_spp_disconnect();
                break;
            }  

            g_print_menu_control_flag_master = TRUE;
        }

        test_rx_process();
    }
}

static bool g_print_menu_control_flag_slave = TRUE;

static void print_menu_command_slave(void)
{
    if(TRUE == g_print_menu_control_flag_slave)
    {
        printf("\n---------------------\n");
        printf("Press menu key - Slave Mode\n");
        printf("---------------------\n");
        printf("0> Reset BT\n");
        printf("---------------------\n");
        printf("1> Normal Mode\n");
        printf("2> Stream Mode\n");
        printf("---------------------\n");
        printf("5> AT CMD: state\n");
        printf("7> AT CMD: firmware version\n");
        printf("---------------------\n");
        printf("8> Send Data: ABCDEFGH01234\n");
        printf("---------------------\n");
        printf("a> AT CMD: discoverable start\n");
        printf("---------------------\n");
    }

    g_print_menu_control_flag_slave = FALSE;
}

void Test_BT200_Slave(void)
{
    uint8_t ch;

    DEBUG_MSG_FUNC_START;
    
    while(1)
    {
        print_menu_command_slave();

        if(FALSE == UartQueue_Serial_Is_Empty())
        {
            ch = UartQueue_Serial_DeQueue();
            printf("%c is selected\n\n", (char)ch);

            switch((char)ch)
            {
            case '0':
                Reset_BT200();
                break;

            case '1':
                ModeChange_PartronBT_to_Normal();
                break;

            case '2':
                ModeChange_PartronBT_to_Stream();
                break;

            case '5':
                make_cmd_module_state();
                break;

            case '7':
                make_cmd_firmware_version();
                break;

            case '8':
                Uart_2_Comm_SendData_Chars("ABCDEFGH01234", sizeof("ABCDEFGH01234"));
                break;

            case 'a':
                make_cmd_discoverable_start();
                break;
            }  

            g_print_menu_control_flag_slave = TRUE;
        }

        test_rx_process();
    }
}

